% Simulate a direct MRAC
% for (q+a)y=bu+(q+c)e

% True system
randn('seed',0);
noiselevel = 0.1;
atrue = -0.9;
btrue = 3;
ctrue = -0.3;
thetatrue = [atrue ;btrue;ctrue];

% Simulate adaptive controller
N = 400;   % nr of samples
time=(1:N)';
wc = 0.05; % square wave reference signal
uc = sign(sin(wc*time));
%uc = randn(size(time)); % better excitation

% initialise vectors
u=zeros(N,1);
y=zeros(N,1);
n=zeros(N,1); 
theta=zeros(3,N);
theta(:,1)=[0;1;0]; % initial guess

phi = zeros(3,N);
K=zeros(3,N);
e=zeros(N,1);
Pinv=zeros(3,3,N);
Pinv(:,:,1)=1e-2*eye(3);

for k=2:N  
  n(k) = noiselevel*randn;
  y(k) = -atrue*y(k-1) + btrue*u(k-1)+n(k)+ctrue*n(k-1);
  
  phi(:,k) = [-y(k-1);  u(k-1); e(k-1)];
  Pinv(:,:,k) = Pinv(:,:,k-1) + phi(:,k)*phi(:,k)';
  K(:,k) = Pinv(:,:,k)\phi(:,k);
  e(k) = y(k)-phi(:,k)'*theta(:,k-1);
  theta(:,k) = theta(:,k-1) + K(:,k)*e(k); 
  a=theta(1,k);
  b=theta(2,k);
  c=theta(3,k);
  
  u(k) =  (a-c)/b*y(k)+1/b*uc(k);     
end

figure(1)
title('parameter estimates')
subplot(311)
plot(theta(1,:));
hold on;
plot([1 N],[thetatrue(1) thetatrue(1)],'r')
text(N/2,-0.5,'a','fontsize',24,'fontweight','b')
subplot(312)
plot(theta(2,:));
text(N/2,2,'b','fontsize',24,'fontweight','b')
hold on;
plot([1 N],[thetatrue(2) thetatrue(2)],'r')
subplot(313)
plot(theta(3,:));
text(N/2,0.25,'c','fontsize',24,'fontweight','b')
hold on;
plot([1 N],[thetatrue(3) thetatrue(3)],'r')


figure(2)
subplot(211)
title('adaptive control of motor')
stairs(y);grid on; hold on
text(N/2,max(y)*0.9,'y','fontsize',24,'fontweight','b')
subplot(212)
stairs(u);grid on; hold on
text(N/2,max(u)*0.8,'u','fontsize',24,'fontweight','b')


