% Simulate a direct MRAC
% for (q+a)y=bu+(q+c)e

% True system
randn('seed',0);
noiselevel = 1;
atrue = -0.9;
btrue = 3;
ctrue = -0.3;
thetatrue = [atrue ;btrue;ctrue];

% Simulate adaptive controller
N = 400;   % nr of samples
%time=(1:N)';
%wc = 0.05; % square wave reference signal
%uc = 0*sign(sin(wc*time));
%uc = randn(size(time)); % better excitation

% initialise vectors
u=zeros(N,1);
y=zeros(N,1);
n=zeros(N,1); 
theta=zeros(3,N);
theta(:,1)=[0;1;0]; % initial guess

phi = zeros(3,N);
K=zeros(3,N);
e=zeros(N,1);
Pinv=zeros(3,3,N);
Pinv(:,:,1)=1e-2*eye(3);

for k=2:N  
  n(k) = noiselevel*randn;
  y(k) = -atrue*y(k-1) + btrue*u(k-1)+n(k)+ctrue*n(k-1);
  
  phi(:,k) = [-y(k-1);  u(k-1); e(k-1)];
  Pinv(:,:,k) = Pinv(:,:,k-1) + phi(:,k)*phi(:,k)';
  K(:,k) = Pinv(:,:,k)\phi(:,k);
  e(k) = y(k)-phi(:,k)'*theta(:,k-1);
  theta(:,k) = theta(:,k-1) + K(:,k)*e(k); 
  a=theta(1,k);
  b=theta(2,k);
  c=theta(3,k);
  
  u(k) =  (a-c)/b*y(k);   % adaptive controller
  %u(k) = (atrue-ctrue)/btrue*y(k);  % ideal minvar control
end



figure(1)
plot(cumsum(abs(y)),'b','linewidth',1.4)
hold on
%plot(cumsum(abs(y)),'r','linewidth',1.4)
legend('Adaptive Control', 'Ideal Min. Var Control','Location','East')

figure(2)
plot((theta(1,:)-theta(3,:))./theta(2,:),'b','linewidth',1.4)
hold on
plot([0 N],(atrue-ctrue)/btrue*[1 1],'r','linewidth',1.4)

